--
-- Debug class for interactive control version 4.0 and above
--
-- Author: Martin Fabík
-- Contact: mar.fabik@gmail.com
--
-- Free for non-comerecial usage
-- 
-- version ID   - 4.0
-- version date - 2017-09-26 14:58:00
--


-- support function
local calculateFinalItemIndex = function(vehicle, itemType, index)
	local calculateIndex=0;
	for i=itemType-1, 0, -1 do
		calculateIndex = calculateIndex + vehicle.LIC.partsLength[i];
	end;
	return calculateIndex+index;
end;

ICDebugGlobal = {};

function ICDebugGlobal:loadMap(name)
end;

function ICDebugGlobal:keyEvent(unicode, sym, modifier, isDown)
end;

function ICDebugGlobal:update(dt)
end;

function ICDebugGlobal:draw()
end;

function ICDebugGlobal:deleteMap()
end;

function ICDebugGlobal:mouseEvent(posX, posY, isDown, isUp, button)
end;

function ICDebugGlobal:gsICPrintStats()
	print("========");
	print("[ INFO ]: Listing IC satistics");
	if g_currentMission.controlledVehicle ~= nil then
		local debugVehicle = nil;
		if g_currentMission.controlledVehicle.selectedImplement == nil then
			debugVehicle = g_currentMission.controlledVehicle;
		else
			debugVehicle = g_currentMission.controlledVehicle.selectedImplement;
		end;
		if debugVehicle ~= nil and debugVehicle.LIC ~= nil then
			print("[ INFO ]:     Founded attacher joints: ");
			print("[ INFO ]:         Front: " .. tostring(#debugVehicle.LIC.joints.front));
			print("[ INFO ]:         Rear : " .. tostring(#debugVehicle.LIC.joints.back));
			print("[ INFO ]:     Available button types and their numbers:");
			for k,v in pairs(InteractiveControl) do
				if type(k) == "string" and k:sub(1,11) == "OBJECT_TYPE" and type(v) == "number" then
					print("[ INFO ]:         Type constant: " .. k);
					print("[ INFO ]:              Index: " .. v);
					print("[ INFO ]:              Registered items: " .. debugVehicle.LIC.partsLength[v]);
					print("[ INFO ]:              First item index offset: " .. calculateFinalItemIndex(debugVehicle, v, 0));
				end;
			end;
		else
			print("[ WARN ]: Selected vehicle/implement don't have an IC included. Please select another vehicle.");
		end;
	else
		print("[ WARN ]: You need to enter or select vehicle before calling gsICPrintStats!");
	end;
	print("========");
end;

function ICDebugGlobal:gsICPrintComponents()
	print("========");
	print("[ INFO ]: Printing IC components");
	if g_currentMission.controlledVehicle ~= nil then
		local debugVehicle = nil;
		if g_currentMission.controlledVehicle.selectedImplement == nil then
			debugVehicle = g_currentMission.controlledVehicle;
		else
			debugVehicle = g_currentMission.controlledVehicle.selectedImplement;
		end;
		if debugVehicle ~= nil and debugVehicle.LIC ~= nil and debugVehicle.LIC_D ~= nil and debugVehicle.LIC_D.actualListingObjectsID ~= nil then
			debugVehicle.LIC_D.actualLocalID = 0;
			local validType = debugVehicle.LIC_D.actualListingObjectsID;
			local validTypeName = nil;
			for k,v in pairs(InteractiveControl) do
				if type(k) == "string" and k:sub(1,11) == "OBJECT_TYPE" and type(v) == "number" then
					if v == validType then
						validTypeName = k;
						break;
					end;
				end;
			end;
			if validTypeName == nil then
				print("[ ERROR ]: An error occured.");
				return;
			end;
			print("[ INFO ]:     Listing statistics for object type " .. tostring(validTypeName));
			print("[ INFO ]:     Objects in this category: " .. tostring(debugVehicle.LIC.partsLength[validType]));
			for k,v in pairs(debugVehicle.LIC.interactiveObjects) do
				if v.objectType == validType then
					local compomentXMLId = k - calculateFinalItemIndex(debugVehicle, validType, 0);
					print("[ INFO ]:         Component global ID: " .. tostring(k));
					print("[ INFO ]:             Component xml ID: " .. tostring(compomentXMLId));
					local xmlName = validTypeName;
					if xmlName == "OBJECT_TYPE_MONITOR_LAYER_BTN" then
						xmlName = "OBJECT_TYPE_MONITOR";
					end;
					print("[ INFO ]:             Component xml type: " .. tostring(validTypeName));
					print("[ INFO ]:             Name: " .. tostring(v.name));
					local indexMessage = v.mark;
					if indexMessage == nil then
						indexMessage = "none - for internal use only";
					end;
					print("[ INFO ]:             i3d index: " .. tostring(indexMessage));
				end;
			end;
			debugVehicle.LIC_D.selectedDebugCategory = debugVehicle.LIC_D.actualListingObjectsID
			local exist = false;
			debugVehicle.LIC_D.actualListingObjectsID = debugVehicle.LIC_D.actualListingObjectsID + 1;
			for k,v in pairs(InteractiveControl) do
				if type(k) == "string" and k:sub(1,11) == "OBJECT_TYPE" and type(v) == "number" then
					if v == debugVehicle.LIC_D.actualListingObjectsID then
						exist = true;
						break;
					end;
				end;
			end;
			if not exist then
				debugVehicle.LIC_D.actualListingObjectsID = 0;
			end;
		else
			print("[ WARN ]: Selected vehicle/implement don't have an IC included. Please select another vehicle.");
		end;
	else
		print("[ WARN ]: You need to enter or select vehicle before calling gsICPrintStats!");
	end;
	print("========");
end;

function ICDebugGlobal:gsICPrintComponent()
	print("========");
	print("[ INFO ]: Printing single IC component");
	if g_currentMission.controlledVehicle ~= nil then
		local debugVehicle = nil;
		if g_currentMission.controlledVehicle.selectedImplement == nil then
			debugVehicle = g_currentMission.controlledVehicle;
		else
			debugVehicle = g_currentMission.controlledVehicle.selectedImplement;
		end;
		if debugVehicle ~= nil and debugVehicle.LIC ~= nil and debugVehicle.LIC_D ~= nil and debugVehicle.LIC_D.selectedDebugCategory ~= nil and debugVehicle.LIC_D.actualLocalID ~= nil then
			local validTypeName = nil;
			for k,v in pairs(InteractiveControl) do
				if type(k) == "string" and k:sub(1,11) == "OBJECT_TYPE" and type(v) == "number" then
					if v == debugVehicle.LIC_D.selectedDebugCategory then
						validTypeName = k;
						break;
					end;
				end;
			end;
			if validTypeName == nil then
				print("[ ERROR ]: An error occured.");
				return;
			end;
			if debugVehicle.LIC.partsLength[debugVehicle.LIC_D.selectedDebugCategory] > 0 then
				local component = debugVehicle.LIC.interactiveObjects[calculateFinalItemIndex(debugVehicle, debugVehicle.LIC_D.selectedDebugCategory, debugVehicle.LIC_D.actualLocalID)];
				if component ~= nil then
					print("[ INFO ]:     Component type: " .. tostring(validTypeName));
					print("[ INFO ]:     Component local ID: " .. tostring(debugVehicle.LIC_D.actualLocalID));
					print("[ INFO ]:     Component global ID: " .. tostring(calculateFinalItemIndex(debugVehicle, debugVehicle.LIC_D.selectedDebugCategory, debugVehicle.LIC_D.actualLocalID)));
					print("[ INFO ]:     Component name: " .. tostring(component.name));
					local indexText = "none - only for internal use";
					if component.mark ~= nil then
						indexText = "set";
					end;
					print("[ INFO ]:     Component index: " .. tostring(indexText));
					print("[ INFO ]:     Component onMessage: " .. tostring(component.onMessage));
					print("[ INFO ]:     Component offMessage: " .. tostring(component.offMessage));
					print("[ INFO ]:     Component size: " .. tostring(component.size));
					print("[ INFO ]:     Component pulsing: " .. tostring(component.pulsingMark));
					print("[ INFO ]:     Component isOutside: " .. tostring(component.canOutside));
					print("[ INFO ]:     Component current status: " .. tostring(component.isOpen));
					print("[ INFO ]:     Component render status text: " .. tostring(component.renderStatusText));
					print("[ INFO ]:     Component synchronized: " .. tostring(component.synch));
					print("[ INFO ]:     Raw component data: ");
					DebugUtil.printTableRecursively(component, "[ INFO ]:         ", 0, 4);
					debugVehicle.LIC_D.actualLocalID = debugVehicle.LIC_D.actualLocalID + 1;
					if debugVehicle.LIC_D.actualLocalID >= debugVehicle.LIC.partsLength[debugVehicle.LIC_D.selectedDebugCategory] then
						debugVehicle.LIC_D.actualLocalID = 0;
					end;
				else
					print("[ ERROR ]: An error occured.");
				end;
			else
				print("[ WARN ]: Selected type (".. tostring(validTypeName) ..") have no components. Select another type via gsICPrintComponents");
			end;
		else
			print("[ WARN ]: Selected vehicle/implement don't have an IC included. Please select another vehicle.");
		end;
	else
		print("[ WARN ]: You need to enter or select vehicle before calling gsICPrintStats!");
	end;
	print("========");
end;

addModEventListener(ICDebugGlobal);

InteractiveControlDebug = {};

if isICDebugLoaded == nil or isICDebugLoaded == false then
	isICDebugLoaded = true;
	addConsoleCommand("gsICPrintStats", "Print IC staticstics of selected vehicle.", "gsICPrintStats", ICDebugGlobal);
	addConsoleCommand("gsICPrintComponents", "Prints all buttons by category.", "gsICPrintComponents", ICDebugGlobal);
	addConsoleCommand("gsICPrintComponent", "Print only one component in selected set.", "gsICPrintComponent", ICDebugGlobal);
end;

function InteractiveControlDebug.prerequisitesPresent(specializations)
	if not SpecializationUtil.hasSpecialization(InteractiveControl, specializations) then
		print("Warning: InteractiveControlDebug class needs InteractiveControl specialization and needs to be loaded after this specialization!");
		return false;
	end;
	return true;
end;

function InteractiveControlDebug:preLoad(savegame)
	print("[ INFO ]: Following vehicle is available for IC Debug.");
	self.LIC_D = {};
	self.LIC_D.selectedDebugCategory = 0;
	self.LIC_D.actualListingObjectsID = 0;
	self.LIC_D.actualLocalID = 0;
	self.LIC_D.actualListenerID = 0;
end;

function InteractiveControlDebug:load(savegame)
end;

function InteractiveControlDebug:postLoad(savegame)
end;

function InteractiveControlDebug:getSaveAttributesAndNodes(nodeIdent)
end

function InteractiveControlDebug:delete()
end;

function InteractiveControlDebug:readStream(streamId, connection)
end;

function InteractiveControlDebug:writeStream(streamId, connection)
end;

function InteractiveControlDebug:mouseEvent(posX, posY, isDown, isUp, button)
end;

function InteractiveControlDebug:keyEvent(unicode, sym, modifier, isDown)
end;

function InteractiveControlDebug:update(dt)
end;

function InteractiveControlDebug:updateTick(dt)
end;

function InteractiveControlDebug:draw()
end;

function InteractiveControlDebug:onEnter()
end;

function InteractiveControlDebug:onLeave()
end;
